/****************************************************************  
* DRIVER PARA EL CONTROL DE PANTALLA LCD HITACHI 44780		*
* Escrito para Microchip XC8/XC16				*
* 	   www.firtec.com.ar informes@firtec.com.ar             *
*****************************************************************/

#include "lcd_xc8.h"

//------------------ Cttes en Flash ------------------------------------------ 
const unsigned char LCD_INIT_STRING[4] =
{
    0x20 | (lcd_type << 2),  // 4 Bits
  	//0x0F,	// Display oncursor ON titilando.
   	0x0C, 	// Display on, cursor off, blink off.
  	1, 	// Clear display.
  	6 	// Increment cursor, shift off.(Entry mode set).
};
/*****************************************************************************
* FUNCION: "void lcd_send_nibble(unsigned char nibble)".
*  OBS: El nibble a enviar debe estar en la parte baja del parmetro "nibble".
******************************************************************************/
void lcd_send_nibble(unsigned char nibble){

	DATA_PIN_4 = !!(nibble & 0x01);	// Genera el valor de cada pin.
	DATA_PIN_5 = !!(nibble & 0x02);
	DATA_PIN_6 = !!(nibble & 0x04);
	DATA_PIN_7 = !!(nibble & 0x08);

	Nop();
	E_PIN = 1;
	Nop();Nop();			// Pulso en "E_PIN".
	E_PIN = 0;
}									
/************************************************************************
* FUNCION: "int8 lcd_read_nibble(void)". 				
*									
*************************************************************************/
#ifdef USE_RW
int8 lcd_read_nibble(void){

	union{
		int8 retval_byte;
		struct{
			int8 retval_0	:1;
			int8 retval_1	:1;
			int8 retval_2	:1;
			int8 retval_3	:1;
		};
	}retval;

	retval.retval_byte = 0;

	E_PIN = 1;
	Nop();
	retval.retval_0 = READ_PIN_4;
	retval.retval_1 = READ_PIN_5;
	retval.retval_2 = READ_PIN_6;
	retval.retval_3 = READ_PIN_7;
	E_PIN = 0;
	return(retval.retval_byte);
}
#endif

/************************************************************************
* FUNCION: "int8 lcd_read_byte(void)". 					
*									
*************************************************************************/
#ifdef USE_RW
int8 lcd_read_byte(void){
	int8 low;
	int8 high;

	TRIS_DATA_PIN_4 = 1;
	TRIS_DATA_PIN_5 = 1;
	TRIS_DATA_PIN_6 = 1;
	TRIS_DATA_PIN_7 = 1;

	RW_PIN = 1;
	Nop();

	high = lcd_read_nibble();
	low = lcd_read_nibble();
	return( (high << 4) | low);
}
#endif

/******************************************************************************
* FUNCION: "void lcd_send_byte(unsigned char address, unsigned char n)". 
*						
* 						
* OBS: 	address = 1 : data.			
*		address = 0 : comando.		
******************************************************************************/ 
void lcd_send_byte(unsigned char address, unsigned char n){

	RS_PIN = 0;

#ifdef USE_RW
	while((lcd_read_byte() & 0x80) == 0x80);
#else
	_delay_100us();
#endif

	TRIS_DATA_PIN_4 = 0;
	TRIS_DATA_PIN_5 = 0;
	TRIS_DATA_PIN_6 = 0;
	TRIS_DATA_PIN_7 = 0;

        if(address)
		RS_PIN = 1;	// Data.
	else
		RS_PIN = 0;    // Comando.
	Nop();
#ifdef USE_RW
	RW_PIN = 0;
	Nop();
#endif

	lcd_send_nibble(n >> 4);		// High nibble.		
	lcd_send_nibble(n & 0xf);  		// Low nibble. 
}

/*****************************************************************************  
* FUNCION: "void lcd_init(void)". 			   
*							
*							
******************************************************************************/
void lcd_init(void){

	unsigned char i;

	TRIS_RS = 0;
	TRIS_E  = 0;
#ifdef USE_RW
	TRIS_RW = 0;
#endif

	RS_PIN = 0;
	E_PIN  = 0;
#ifdef USE_RW
	RW_PIN = 0;
#endif

	TRIS_DATA_PIN_4 = 0;
	TRIS_DATA_PIN_5 = 0;
	TRIS_DATA_PIN_6 = 0;
	TRIS_DATA_PIN_7 = 0;

	_delay_5ms();			
	_delay_5ms();
	_delay_5ms();
	for(i = 0; i < 3; i++){
		lcd_send_nibble(0x03);
		_delay_5ms();
  	}

	lcd_send_nibble(0x02);	// 4-bit interface.

	for(i = 0; i < sizeof(LCD_INIT_STRING); i++){
		lcd_send_byte(0, LCD_INIT_STRING[i]);
		#ifndef USE_RW
			_delay_5ms();
		#endif
	}
}

/*****************************************************************************   
* FUNCION: "void lcd_gotoxy(unsigned char x, unsigned char y)". 
* 								
* x: Columna.							
* y: Fila. 							
******************************************************************************/
void lcd_gotoxy(unsigned char x, unsigned char y){

	unsigned char address;	// Line 1: 0x00..0x27. Line 2: 0x40..0x67.

	if(y != 1)
		address = lcd_line_two;
	else
		address = 0;

	address += x-1;
	lcd_send_byte(0, 0x80 | address); // Set DD RAM address.
}
/***************************************************************************** 
* FUNCION: "void lcd_putc(unsigned char c)". 				
*									
*									
*****************************************************************************/
void lcd_putc(unsigned char c){

	switch(c){
		case '\f':
			lcd_send_byte(0,1);	
			break;
		case '\n':			// ENTER.
			lcd_gotoxy(1,2);// Posiciona cursor en lnea 2.
			break;
		case '\b':			// BACKSPACE.
      		lcd_send_byte(0,0x10);  
			break;
		default:
			lcd_send_byte(1,c);  // Send char.
			break;
  	}
}
/**************************************************************************
* FUNCION: "void lcd_comand(unsigned char d)". 				
* Enva el comando "d" al LCD.						
*									
***************************************************************************/
void lcd_comand(unsigned char d){
	lcd_send_byte(0,d);
}

/****************************************************************************
* FUNCION: "int8 lcd_getc(int8 x, int8 y)". 
*						
*					
*****************************************************************************/
#ifdef USE_RW
int8 lcd_getc(int8 x, int8 y){

	int8 value;

	lcd_gotoxy(x,y);

	while((lcd_read_byte()& 0x80)== 0x80);

	RS_PIN = 1;
	value  = lcd_read_byte();
	RS_PIN = 0;
	return(value);
}
#endif

/******************************************************************************
* FUNCION: "void lcd_setcursor_vb(unsigned char visible, unsigned char blink)".
* Enva el comando "Display ON/OFF control".		
*******************************************************************************/ 
void lcd_setcursor_vb(unsigned char visible, unsigned char blink) {
	lcd_send_byte(0, 0xC | (visible << 1) | blink);	
}
/******************************************************************************
* FUNCION: "void lcd_puts(char *ptr)". 	
* LCD put String(RAM).			
*					
* PREGUNTA:  Sale cuando llega al NULL del final del STRING ?
******************************************************************************/
void lcd_puts(char *ptr){

	while(*ptr){		
		lcd_putc(*ptr++);
	}
}
/******************************************************************************
* FUNCION: "void lcd_putrs(const rom char *ptr)". 
* LCD put ROM String (Flash).						
******************************************************************************/
void lcd_putrs(const char *ptr){
	
	while(*ptr){
		lcd_putc(*ptr++);
	}
}
